#        *         *         *         *         *         *         *         #
# Metadata                                                                  ####
#
# Title:    sparese RDA
# meta:     sparese RDA
#
# code by:  Attila Csala
# place:    Amsterdam
# date:     2016-11
#
# setwd("C:/amc.intra/users/A/acsala/home/sRDA/")
# devtools::load_all()
# devtools::use_package_doc()
# devtools::document()
# devtools::check()
# devtools::use_vignette("sRDA")
# devtools::build_vignettes()
# devtools::build()
# devtools::install()
# install.packages("C:/amc.intra/users/A/acsala/home/sRDA_0.1.tar.gz")
#        *         *         *         *         *         *         *         #

#' Sparse Redundancy Analysis
#'
#' Sparse Redundancy Analysis to express the maximum variance in the predicted
#' data set by a linear combination of variables of the predictive data set
#' 
#'
#'
#' 
#' @param predictor 
#' The n*p matrix of the predictor data set
#' @param predicted 
#' The n*q matrix of the predicted data set
#' @param penalization 
#' The penalization method applied during the analysis (none, enet or ust)
#' @param ridge_penalty 
#' The ridge penalty parameter of the predictor set's latent variable used 
#' for enet or ust (an integer if cross_validate = FALE, a list otherwise)
#' @param nonzero 
#' The number of non-zero weights of the predictor set's latent variable 
#' (an integer if cross_validate = FALE, a list otherwise)
#' @param max_iterations 
#' The maximum number of iterations of the algorithm
#' @param tolerance 
#' Convergence criteria
#' @param cross_validate 
#' K-fold cross validation to find best optimal penalty parameters 
#' (TRUE or FALSE)
#' @param parallel_CV
#' Run the cross validation parallel (TRUE or FALSE)
#' @param nr_subsets
#' Number of subsets for k-fold cross validation
#' @param multiple_LV
#' Obtain multiple latent variables (TRUE or FALSE)
#' @param nr_LVs
#' Number of latent variables to be obtained
#' 
#' @return An object of class \code{"srda"}.
#' @return \item{ALPHA}{
#' Weights of the predictor set's latent variable}
#' @return \item{BETA}{
#' Weights of the predicted set's latent variable}
#' @return \item{Nr_iterations}{
#' Number of iterations the algorithm did until convergence}
#' @return \item{XI}{
#' Predictor set's latent variable scores}
#' @return \item{ETA}{
#' Predictive set's latent variable scores}
#' @return \item{SOLVE_XIXI}{
#' Inverse of the predictor set's latent variable variance matrix}
#' 
#' @author Attila Csala
#' @keywords Redundancy RDA
#' @import elasticnet
#' @import mvtnorm
#' @import parallel
#' @import doParallel
#' @import Matrix
#' @export
#' @examples
#' \dontrun{
#'
#' # generate data
#' dataXY <- generate_data(number_of_ksi = 1,
#'                            number_of_patients = 500,
#'                            number_of_Xs_associated_with_ksis = c(5),
#'                            number_of_not_associated_Xs = 250,
#'                            mean_of_the_regression_weights_of_the_associated_Xs =
#'                              c(0.9),
#'                            sd_of_the_regression_weights_of_the_associated_Xs =
#'                              c(0.05),
#'                            Xnoise_min = -0.3, Xnoise_max = 0.3,
#'                            
#'                            number_of_Ys_associated_with_ksis = c(5),
#'                            number_of_not_associated_Ys = 350,
#'                            mean_of_the_regression_weights_of_the_associated_Ys =
#'                              c(0.9),
#'                            sd_of_the_regression_weights_of_the_associated_Ys =
#'                              c(0.05),
#'                            Ynoise_min = -0.3, Ynoise_max = 0.3)
#'
#' # seperate predictor and predicted sets
#' X <- dataXY$X
#' Y <- dataXY$Y
#'
#' # run sRDA
#' RDA.res <- sRDA(predictor = X, predicted = Y, nonzero = 5, 
#' ridge_penalty = 1, penalization = "ust")
#' 
#' # check first 10 weights of X
#' RDA.res$ALPHA[1:10]
#' 
#' # run sRDA with cross-validation to determine best penalization parameters
#' RDA.res <- sRDA(predictor = X, predicted = Y, nonzero = c(5,10,15), 
#' ridge_penalty = c(0.1,1), penalization = "enet", cross_validate = TRUE,
#' parallel_CV = TRUE)
#' 
#' # check first 10 weights of X
#' RDA.res$ALPHA[1:10]
#' RDA.res$ridge_penalty
#' RDA.res$nr_nonzeros
#' 
#' # obtain multiple latent variables
#' RDA.res <- sRDA(predictor = X, predicted = Y, nonzero = c(5,10,15), 
#' ridge_penalty = c(0.1,1), penalization = "enet", cross_validate = TRUE, 
#' parallel_CV = TRUE, multiple_LV = TRUE, nr_LVs = 3, max_iterations = 5)
#' 
#' # check first 10 weights of X in first two component
#' RDA.res$ALPHA[[1]][1:10]
#' RDA.res$ALPHA[[2]][1:10]
#' 
#' # scores are orthogonal to each other
#' t(RDA.res$XI[[1]]) %*% RDA.res$XI[[2]]
#' 
#' }

#******************************************************************************#
#                               Peanalized RDA                              ####
#******************************************************************************#
#+-------+                                                            +-------+
#|       |                                                            |       |
#|       |                                                            |       |
#|       |                                                            |       |
#|       |   ALPHA       +------+          +------+       BETA        |       |
#|       +--------------->      |          |      +------------------->       |
#|       |               |      |          |      |                   |       |
#|       |               |      |          |      |                   |       |
#|  X    +--------------->  XI  |          | ETA  +------------------->   Y   |
#|       |               |      +---------->      |                   |       |
#|       |               |      |          |      |                   |       |
#|       +--------------->      |          |      +------------------->       |
#|       |               +------+          +------+                   |       |
#|       |                                                            |       |
#|       |                                                            |       |
#|       |                                                            |       |
#+-------+                                                            +-------+

#function for peanalized redundancy analysis
#
#input:     X             (n*p matrix     - predictor variables),
#           Y             (n*q matrix     - predicted variables),
#           ridge_penalty (integer        - factor for Ridge penalty)
#           nonzero       (integer        - factor for LASSO penalty)
#
#output:    ALPHA         p*1 vector      #ALPHA   = X-WEIGHTS
#           BETA,         q*1 vector      #BETA     = Y-WEIGHTS
#           XI,           n*1 vector      #XI       - latent variable for X
#           ETA           n*1 vector      #ETA      - latent variable for Y

sRDA <- function(predictor,
                 predicted,
                 penalization = "enet",
                 ridge_penalty = 1,
                 nonzero = 1,
                 #stop criteruim in a form of maximum iterations
                 max_iterations = 100,
                 #stop criterium regarding to CRT
                 tolerance = 1*10^-25,
                 cross_validate = FALSE,
                 parallel_CV = TRUE,
                 nr_subsets = 10,
                 multiple_LV = FALSE,
                 nr_LVs = 1
                 ){

  #****************************************************************************#
  #0. ancillary functions
  #*********************
  #check for dependencies
  if (!requireNamespace("elasticnet", quietly = TRUE)) {
        stop("elasticnet needed for this function to work. Please install it.",
             call. = FALSE)
  }
  
  #Check for cross validation *************************************************#
  
  if (multiple_LV){
    
    result <- get_multiple_LVs(X = predictor,
                               Y = predicted,
                               penalization = penalization,
                               lambda = ridge_penalty,
                               nonzero = nonzero,
                               nr_latent = nr_LVs,
                               stop_criterium = tolerance,
                               max_iterations = max_iterations)
    
    return(result)
    
    
  }
  
  if (cross_validate){
    cross_validation_results <- 
      get_cross_validated_penalty_parameters(predictor = predictor,
                                             predicted = predicted,
                                             penalization = penalization,
                                             ridge_penalty = ridge_penalty,
                                             nonzero = nonzero,
                                             nr_subsets = nr_subsets,
                                             max_iterations = max_iterations,
                                             tolerance = tolerance,
                                             parallel_CV = parallel_CV
                                             )
    ridge_penalty   <- cross_validation_results$best_ridge
    nonzero         <- cross_validation_results$best_nonzero
  }
  

  #Algorithm for peanalization*************************************************#

  #****************************************************************************#
  #1. Preperation of the data
  #*************************#

  Y.mat <- as.matrix(predicted)
  #scale (SD = 1) and center (mean = 0) Y
  Yc <- scale(Y.mat)
  Yc <- Yc[,!colSums(!is.finite(Yc))]

  X.mat <- as.matrix(predictor)
  #scale (SD = 1) and center (mean = 0) X
  Xcr <- scale(X.mat)
  Xcr <- Xcr[,!colSums(!is.finite(Xcr))]

  #variable length of X and Y
  p <- ncol(Xcr)
  q <- ncol(Yc)

  #   For initalization, let:
  #
  #         ETA^(0) = y_1 + y_2 + .... + y_q = Y*BETA_hat^(0) ( = Y_i)
  #           were BETA^(0) = [1,1....,1]
  # an y-weight column vector of q elements
  BETA = matrix( rep(1, q), nrow = q, ncol = 1, byrow = TRUE)
  # an x-weight column vector of p elements
  ALPHA = matrix( rep(1, p), nrow = p, ncol = 1, byrow = TRUE)


  #****************************************************************************#
  #2. Iterative loop until convergence
  #*************************#
  CRTs          <- c()
  sum_abs_Betas <- c()
  Nr_iterations = 0         #iteration counter

  WeContinnue = TRUE        #T/F value for stop criterium based on CRT values
                            #in last two iteration
  CRT = 1                   #convergance measure between alpha and beta

  while(CRT > tolerance && WeContinnue && Nr_iterations < max_iterations) {

    ETA = Yc %*% BETA
    ETA = scale(ETA)


    XI = Xcr %*% ALPHA
    XI = scale(XI)


    #**************************************************************************#
    #Chose between generic RDA, SRDA with ENET or SRDA wiht UST****************#

    # For the value of ahat^(0) (ALPHA) and hence XIhat^(0),
    # regress ETAhat^(0) on X jointly to get ALPHA,
    # ALPH_0 = (X'X)^-1 (X' ETA)

    ALPH_0 <- switch(penalization,
                #lm without penalization
                #ALPH_0 = as.matrix(lm(ETA ~ 0+Xcr)$coefficients)
                "none" = {
                  solve(t(Xcr) %*% Xcr) %*% t(Xcr) %*% ETA
                  },

                #CALCULATE WITH ELASTIC NET
                #ALPH_0 = as.matrix(calculateVectorEnet(Xcr,ETA,1,30))
                "enet" = {
                  as.matrix(get_enet(Xcr,ETA,ridge_penalty,nonzero))
                  },

                #CALCULATE WITH UST
                #ALPH_0 = as.matrix(calculateVectorEnet(Xcr,ETA,1,30))
                "ust" = {
                  as.matrix(get_ust(Xcr,ETA,nonzero))
                  },
                {
                  stop("Please choose a valid penalization method
                    (i.e. 'ust', 'enet' or 'none')",
                       call. = FALSE)

                }
                
                )
    
    #***************************************************************************


    #!THIS FORM OF NORMALIZATION BRAKES PROPERTY cor(XI,Yc) == t(BETA) !***
    #normalize ALPHA
    #ALPH4_0 = (as.numeric(ALPH_0) %*% solve(sqrt(t(ALPH_0) %*%
    # cov(Xcr,Xcr) %*% ALPH_0 )))
    #ALPH4_0 = (as.numeric(ALPH_0) %*% solve(sqrt(t(ALPH_0) %*%
    # t(Xcr) %*% Xcr %*% ALPH_0 )))

    #is t(ahat^(0)) t(X)*X*ahat^(0) = 1 /normalized?
    #t(ALPH4_0) %*%  t(Xcr) %*% Xcr %*% ALPH4_0
    #***************************************************************************



    #         compute the value of XI^(1):
    #         XIhat^(1) = SUM_running to p where t=1 ( ahat_t^(0) *x_t )
    
    XI = Xcr %*% ALPH_0

    #dim(Xcr);dim(ALPH_0);dim(XI)

    #         and normalize XIhat^(1) such that
    #           t(XIhat^(1))*XIhat^(1) = t(ahat^(0)) t(X)*X*ahat^(0) = 1
    #           t(XI)%*%XI = t(ALPH_0) %*% t(Xcr) %*% Xcr %*% ALPH_0 = 1
    #           that is its variance is 1
    XI = scale(XI)


    #         For the value BETAhat^(1) and hence ETAhat^(1),
    #           regress y1,y2 ... yq separately on XIhat^(1),
    #
    #           y1 = BETA_1 * XIhat^(1) + Epsilon_1
    #           .
    #           yq = BETA_q * XIhat^(1) + Epsilon_q

    BETA_0 = solve(t(XI)%*%XI) %*% t(XI) %*% Yc
    BETA_0 = t(as.matrix(BETA_0))

    #BETA
    #         compute the vaule of ETAhat^(1),
    #
    #           ETAhat^(1) = SUM_running to q where k=1 ( BETAhat_k^(1) * y_k)
    ETA = Yc %*% BETA_0

    ETA = scale(ETA)

    #check if ETA needed to be scaled here <- nope
    #ETA = scale(ETA)


    #Calculate convergence of Alphas and Betas
    CRT = sum((ALPHA - ALPH_0)^2, (BETA - BETA_0)^2);


    ALPHA=            ALPH_0
    BETA =            BETA_0

    #Check if last two iterations CR converges*********************************#
    Nr_iterations                   =   Nr_iterations + 1
    CRTs[[Nr_iterations]]           =   CRT
    sum_abs_Betas[[Nr_iterations]]  =   sum(abs(BETA))

    if (Nr_iterations>1){

      stop_condition <- abs(CRTs[[Nr_iterations]] - CRTs[[Nr_iterations-1]])
      stop_criterium <- 1 * 10^-6

      if (stop_condition < stop_criterium){
        WeContinnue <- FALSE
      }

    }#END Check if last two iterations CR converges*************************#

  }# End of main loop

  #plot(CRTs[2:length(CRTs)])

  #covariance matrices

  #  n = nrow(Yc)

  #  XXMA = (1/(n-1)) * t(Xcr) %*% Xcr
  #  XYMA = (1/(n-1)) * t(Xcr) %*% Yc
  #  YYMA = (1/(n-1)) * t(Yc) %*% Yc

  #XLDG = X-LOADINGS / eigenvectors / site scores
  #  XLDG = XXMA %*% ALPHA

  #  t(XLDG) %*% XLDG

  #YWGT = Y-WEIGHTS / ginverse
  #  YWGT = BETA %*% solve(sqrt(t(BETA) %*% YYMA %*% BETA))

  #YLDG = Y-LOADINGS
  #  YLDG = YYMA %*% YWGT

  #PATH = PATH COEFFICIENT / correlation between XI and ETA
  #  PATH = t(ALPHA) %*% XYMA %*% YWGT;

  #REDN = REDUNDANCY INDEX: redundancy of y variables
  #  REDN = (sum(BETA^2))/4
  Red_index = (sum(BETA^2))/4

  #************************

  #  t(BETA) %*% t(XYMA) %*% solve(XXMA) %*% XYMA %*% BETA

  #  PATH


  #***************************

  #use this later for second latent variable
  SOLVE_XIXI = solve(t(XI)%*%XI) %*% t(XI)

  result <- list(ALPHA = ALPHA,    #ALPHA= X-WEIGHTS
                 BETA= BETA,    #BETA = Y-CROSSLOADINGS
                 nr_iterations = Nr_iterations,
                 XI = XI,      #XI
                 ETA = ETA,
                 inverse_of_XIXI = SOLVE_XIXI, #solve(t(XI)%*%XI) %*% t(XI)
                 iterations_crts = CRTs,
                 redundancy_index = Red_index,
                 sum_absolute_betas = sum_abs_Betas,
                 ridge_penalty = ridge_penalty,
                 nr_nonzeros = nonzero,
                 nr_latent_variables = nr_LVs
  )

  class(result) = "sRDA"
  result

}
